/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package mx4j.connector;

import java.rmi.RemoteException;
import java.util.Set;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.IntrospectionException;
import javax.management.InvalidAttributeValueException;
import javax.management.ListenerNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanRegistrationException;
import javax.management.NotCompliantMBeanException;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.ObjectInstance;
import javax.management.ObjectName;
import javax.management.QueryExp;
import javax.management.ReflectionException;

/**
 * A restricted MBeanServer interface to be used by RMI clients. <p>
 * For now is exactly equal to MBeanServer, but
 * <ul>
 * <li> instantiate returns the MBean without registering it, has no sense remotely
 * <li> deserialize makes no sense used remotely and should be removed
 * </ul>
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.4 $
 */
public interface RemoteMBeanServer extends MBeanServerConnection
{
	public void addNotificationListener(ObjectName observed, NotificationListener listener, NotificationFilter filter, Object handback)
			throws InstanceNotFoundException, RemoteException;

	public void removeNotificationListener(ObjectName observed, NotificationListener listener, NotificationFilter filter, Object handback)
			throws InstanceNotFoundException, ListenerNotFoundException, RemoteException;

	public ObjectInstance createMBean(String className, ObjectName objectName)
			throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, RemoteException;

	public ObjectInstance createMBean(String className, ObjectName objectName, ObjectName loaderName)
			throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException, RemoteException;

	public ObjectInstance createMBean(String className, ObjectName objectName, Object[] args, String[] parameters)
			throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, RemoteException;

	public ObjectInstance createMBean(String className, ObjectName objectName, ObjectName loaderName, Object[] args, String[] parameters)
			throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException, RemoteException;

	public void unregisterMBean(ObjectName objectName)
			throws InstanceNotFoundException, MBeanRegistrationException, RemoteException;

	public Object getAttribute(ObjectName objectName, String attribute)
			throws MBeanException, AttributeNotFoundException, InstanceNotFoundException, ReflectionException, RemoteException;

	public void setAttribute(ObjectName objectName, Attribute attribute)
			throws InstanceNotFoundException, AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException, RemoteException;

	public AttributeList getAttributes(ObjectName objectName, String[] attributes)
			throws InstanceNotFoundException, ReflectionException, RemoteException;

	public AttributeList setAttributes(ObjectName objectName, AttributeList attributes)
			throws InstanceNotFoundException, ReflectionException, RemoteException;

	public Object invoke(ObjectName objectName, String methodName, Object[] args, String[] parameters)
			throws InstanceNotFoundException, MBeanException, ReflectionException, RemoteException;

	public String getDefaultDomain() throws RemoteException;

	public Integer getMBeanCount() throws RemoteException;

	public boolean isRegistered(ObjectName objectname) throws RemoteException;

	public MBeanInfo getMBeanInfo(ObjectName objectName)
			throws InstanceNotFoundException, IntrospectionException, ReflectionException, RemoteException;

	public ObjectInstance getObjectInstance(ObjectName objectName)
			throws InstanceNotFoundException, RemoteException;

	public boolean isInstanceOf(ObjectName objectName, String className)
			throws InstanceNotFoundException, RemoteException;

	public Set queryMBeans(ObjectName patternName, QueryExp filter) throws RemoteException;

	public Set queryNames(ObjectName patternName, QueryExp filter) throws RemoteException;
}
